unit hiKnob;

interface

uses Windows,Kol,Share,Debug,Win,Messages;

type
  THIKnob = class(THIWin)
   private
    fcaption: string;
    fmarkmove: Boolean;  //  mousemove
    fposition: Integer;
    fMin: Integer;
    fMax: Integer;
    fPageSize: Word;
    fDiameter: Integer;
    fRadius: Integer;
    fSteps: Integer;
    fAngleInterval: Single;
    fAngle: Integer;
    fMouseAngle: Integer;
    fDragging: Boolean;
    fIndicatorInnerPosition: TPoint;
    fIndicatorOuterPosition: TPoint;
    fLastIndicatorInnerPosition: TPoint;
    fLastIndicatorOuterPosition: TPoint;
    procedure SetMin(const NewMinValue: Integer);
    procedure SetMax(const NewMaxValue: Integer);
    procedure SetParams(APosition, AMin, AMax, ASet: Integer);
    procedure SetSteps;
    procedure CalcAngle;
    function  CalcPosition(const TheAngle: Integer): Integer;
    property Position: Integer read fposition write fposition;

   protected
    function  _onMessage( var Msg: TMsg; var Rslt: Integer ): Boolean;
    procedure Paint( Sender: PControl; DC: HDC );
    procedure PaintKnob;
    procedure PaintState;
    procedure PaintIndicator;

   public
    _prop_MinValue:integer;
    _prop_MaxValue:integer;
    _prop_Position:integer;
    _prop_Steps:integer;
    _prop_PageSize:integer;
    _prop_CaptionVal:string;

    _event_onPosition:THI_Event;
    _event_onCaptionPos:THI_Event;

    destructor Destroy; override;
    procedure Init;override;
    procedure _work_doPosition(var _Data:TData; Index:word);
    procedure _work_doPosition2(var _Data:TData; Index:word);
    procedure _work_doCaptionVal(var _Data:TData; Index:word);
    procedure _work_MinValue(var _Data:TData; Index:word);
    procedure _work_MaxValue(var _Data:TData; Index:word);
    procedure _var_Position(var _Data:TData; Index:word);

    property MaxValue: Integer read fMax write SetMax;
    property MinValue: Integer read fMin write SetMin;
    //property Position: Integer read fposition write fposition;
    property PageSize: Word read fPageSize write fPageSize default 10;
  end;

implementation


function THIKnob._onMessage;
var
  NewPosition: Integer;
  H:Integer; dt:Tdata;
begin
  Result:=false;
  case msg.message of
  WM_CREATE:
     begin
       SetSteps;
       CalcAngle;
     end;
  WM_SIZE:
     begin
       if (Control.Height > fDiameter) or (Control.Width > fDiameter)
         then H := Max(Control.Height, Control.Width) else
       if (Control.Height < fDiameter) or (Control.Width < fDiameter)
         then H := Min(Control.Height, Control.Width)
       else H := fDiameter;
       fDiameter := H;
       fRadius := H div 2;
    end;
  WM_SETFOCUS:
      PaintState;
  WM_KILLFOCUS:
     begin
       fDragging := False;
       PaintState;
     end;
  WM_MOUSEMOVE: begin
        fmarkmove := not fmarkmove;
        //dtInteger(dt, fposition);
        _hi_CreateEvent(dt,@_event_onPosition,fposition);
        if (msg.wParam = VK_LBUTTON) and (fmarkmove = true) then
        begin
          if loword(msg.lparam) = Control.Width div 2 then
          begin
            if hiword(msg.lParam) > Control.Height div 2
            then fMouseAngle := 240
            else fMouseAngle := 90;
          end
          else
          begin
            fMouseAngle := Round(ArcTan(((Control.Height div 2)-hiword(msg.lparam))/(loword(msg.lparam)-(Control.Width div 2)))*180/pi);
            if loword(msg.lparam) < Control.Width div 2 then fMouseAngle := (fMouseAngle + 540) MOD 360;
          end;

        if (fMouseAngle <= 270) and (fMouseAngle > 240) then fMouseAngle := 240;
        if (fMouseAngle < -60) then fMouseAngle := -60;

        NewPosition := CalcPosition(fMouseAngle);
        if fDragging and (fposition <> NewPosition) then Position:= NewPosition;
        SetParams(NewPosition,fMin,fMax,1);
      end;
    end;
  WM_KEYDOWN:
     begin
       case msg.wparam  of
       VK_PRIOR : begin Position:= Position + fPageSize; SetParams(Position,fMin,fMax,1);end;
       VK_NEXT  : begin Position:= Position - fPageSize; SetParams(Position,fMin,fMax,1);end;
       VK_END   : begin Position:= fMin; SetParams(Position,fMin,fMax,1);end;
       VK_HOME  : begin Position:= fMax; SetParams(Position,fMin,fMax,1);end;
       VK_LEFT  : begin Position:= Position -1; SetParams(Position,fMin,fMax,1);end;
       VK_UP    : begin Position:= Position +1; SetParams(Position,fMin,fMax,1);end;
       VK_RIGHT : begin Position:= Position +1; SetParams(Position,fMin,fMax,1);end;
       VK_DOWN  : begin Position:= Position -1; SetParams(Position,fMin,fMax,1);end;
       end;
        PaintIndicator;
     end;
  WM_LBUTTONDOWN:
      begin
        fDragging := True;
        PaintIndicator;
        _hi_onEvent(_event_onPosition,Position);
      end;
  WM_LBUTTONUP:
    if fDragging then
    begin
      _hi_onEvent(_event_onPosition,Position);
      fDragging := False;
    end;
  WM_MOUSEWHEEL:
    begin
      case msg.wparam  of   //const ???
         7864320: begin Position:= Position +1; SetParams(Position,fMin,fMax,1);end;
        -7864320: begin Position:= Position -1; SetParams(Position,fMin,fMax,1);end;
      end;
    end;
  else
    begin
      Result:=false;
    end;
  end;
end;

procedure THIKnob.Paint;
begin
  PaintKnob;
  PaintState;
  PaintIndicator;
end;

procedure THIKnob.SetMin(const NewMinValue: Integer);
begin
  SetParams(fposition, NewMinValue, fMax,1);
end;

procedure THIKnob.SetMax(const NewMaxValue: Integer);
begin
  SetParams(fposition, fMin, NewMaxValue,1);
end;

procedure THIKnob.SetParams(APosition, AMin, AMax, ASet: Integer);
begin
  if (fMin <> AMin) then
  begin
    fMin := AMin;
    SetSteps;
  end;

  if (fMax <> AMax) then
  begin
    fMax := AMax;
    SetSteps;
  end;

  if fAngleInterval >= 0 then
  begin
    if APosition < AMin then APosition := AMin;
    if APosition > AMax then APosition := AMax;
  end else
  begin
    if APosition > AMin then APosition := AMax;
    if APosition < AMax then APosition := AMin;
  end;

  if fposition <> APosition then Position := APosition;

  CalcAngle;
  PaintIndicator;
  if ASet = 1 then  _hi_onEvent(_event_onPosition,APosition);
  _hi_onEvent(_event_onCaptionPos, fcaption + int2str(APosition));
end;

procedure THIKnob.SetSteps;
begin
  fSteps := fMax - fMin;
  if fSteps = 0 then fAngleInterval:= 0 else
  begin
    fAngleInterval := 300 / fSteps;
    fSteps := Abs(fSteps);
  end;
end;

procedure THIKnob.CalcAngle;
begin
  fAngle := 240 - Round((fposition - fMin) * fAngleInterval);
end;

function THIKnob.CalcPosition(const TheAngle: Integer): Integer;
begin
  if fAngleInterval = 0 then Result := fMin else
    Result := fMin + Round((240 - TheAngle) / fAngleInterval);
end;

procedure THIKnob.PaintIndicator;
var
  AngleInRadians, CosAngle, SinAngle: Single;
begin
  AngleInRadians := fAngle*pi/180;
  CosAngle := Cos(AngleInRadians);
  SinAngle := Sin(AngleInRadians);

  with Control.Canvas^ do
  begin
    Pen.penwidth:=2;
    Pen.Color := Control.Color;
    with fIndicatorInnerPosition do MoveTo(X+1,Y+1);
    with fIndicatorOuterPosition do LineTo(X+1,Y+1);
    PolyLine([fIndicatorInnerPosition, fIndicatorOuterPosition]);

    fLastIndicatorInnerPosition := fIndicatorInnerPosition;
    fLastIndicatorOuterPosition := fIndicatorOuterPosition;

    fIndicatorInnerPosition.X := fRadius + Round(((fRadius)-11)*CosAngle);
    fIndicatorInnerPosition.Y := fRadius - Round(((fRadius)-11)*SinAngle);

    fIndicatorOuterPosition.X := fRadius + Round(((fRadius)-5)*CosAngle);
    fIndicatorOuterPosition.Y := fRadius - Round(((fRadius)-5)*SinAngle);

    Pen.Color := Control.Color2;
    with fIndicatorInnerPosition do MoveTo(X+1,Y+1);
    with fIndicatorOuterPosition do LineTo(X+1,Y+1);

    Pen.Color := Control.Color1;
    PolyLine([fIndicatorInnerPosition, fIndicatorOuterPosition]);
  end;
end;

procedure THIKnob.PaintState;
begin
  with Control.Canvas^, Control.ClientRect do
  begin
    Pen.penWidth:= 1;
    Brush.BrushStyle:= bsClear;
    if Control.Enabled then Pen.Color := clBlack else Pen.Color := Control.Color1;
    Ellipse(0, 0, Control.Width, Control.Height);
    if Control.focused then Pen.Color := clBlack else Pen.Color := Control.Color1;
    Arc(Left,Top,Right,Bottom,Right,Top,Left,Bottom);
  end;
end;

procedure THIKnob.PaintKnob;
begin
  with Control.Canvas^, Control.ClientRect do
  begin
    Pen.Color := Control.Color;
    Brush.Color := Control.Color;
    Brush.BrushStyle := bsSolid;

    Ellipse(2, 2, Control.Width-2, Control.Height-2);
    Pen.penWidth := 1;
    Pen.Color := Control.Color2;
    Arc(Left+2,Top+2,Right,Bottom,Right,Top,Left,Bottom);

    Pen.Color := Control.Color1;
    Arc(Left+1,Top+1,Right-1,Bottom-1,Left,Bottom,Right,Top);
  end;
end;

procedure THIKnob.Init;
begin
  Control := NewBitbtn(FParent,'',[bboNoBorder],
             GlyphOver,0,0);//.setsize(_prop_Width,_prop_Height);
  Control.Color:=clbtnFace;
  Control.Color1:=clBtnShadow;
  Control.Color2:=clBtnHighLight;
  fRadius := Control.width div 2;
  fDragging:=false;
  //fPageSize:= Pagesize;
  fPageSize:= 10;
  fMouseAngle:=240;
  fcaption:= _prop_CaptionVal;
  fMin:= _prop_MinValue;
  fMax:= _prop_MaxValue;
  Position:= _prop_Position;
  SetSteps;
  CalcAngle;
  SetParams(Position,fMin,fMax,1);
  Control.Transparent:=true;
  Control.OnPaint := Paint;
  Control.OnMessage := _OnMessage;
  inherited;
end;

destructor THIKnob.Destroy;
begin
   //Control.Free;
   inherited Destroy;
end;

procedure THIKnob._work_doPosition;
begin
  SetSteps;
  CalcAngle;
  SetParams(ToInteger(_Data),fMin,fMax,1);
end;

procedure THIKnob._work_doPosition2;
begin
  SetSteps;
  CalcAngle;
  SetParams(ToInteger(_Data),fMin,fMax,0);
end;

procedure THIKnob._work_MinValue;
begin
  fMin:= ToInteger(_Data);
  SetSteps;
  CalcAngle;
  SetParams(Position,fMin,fMax,1);
end;

procedure THIKnob._work_MaxValue;
begin
  fMax:= ToInteger(_Data);
  SetSteps;
  CalcAngle;
  SetParams(Position,fMin,fMax,1);
end;

procedure THIKnob._work_doCaptionVal;
begin
  fcaption:= ToString(_Data);
end;

procedure THIKnob._var_Position;
var dt: TData;
begin
  dtInteger(dt, Position);
end;

end.
